@@ -17,10 +17,11 @@ import errno
 import os
 import select
 import stat
-import subprocess
 import sys
 import time
-import six
+import docker
+import docker.errors
+from docker.types import Mount as DockerMount
 
 import eventlet
 import json
@@ -214,9 +215,12 @@ class RunTimeSandbox(object):
         self.paths = RunTimePaths(scope, conf)
         self.scope = scope
 
-        self.sandbox_ping_interval = 0.5
+        self.sandbox_ping_interval = \
+            float(conf.get('sandbox_ping_interval', 0.5))
+        self.sandbox_stop_timeout = \
+            float(conf.get('stop_linux_container_timeout', 1))
         self.sandbox_wait_timeout = \
-            int(conf.get('restart_linux_container_timeout', 10))
+            float(conf.get('restart_linux_container_timeout', 10))
 
         self.docker_repo = conf.get('docker_repo', '')
         self.docker_image_name_prefix = 'tenant'
@@ -232,7 +236,10 @@ class RunTimeSandbox(object):
 
         self.default_docker_image_name = \
             conf.get('default_docker_image_name',
-                     'ubuntu_18.04_jre11_storlets')
+                     'storlet_engine_image')
+
+        self.max_containers_per_node = \
+            int(conf.get('max_containers_per_node', 0))
 
     def ping(self):
         """
@@ -290,38 +297,53 @@ class RunTimeSandbox(object):
         docker_container_name = '%s_%s' % (self.docker_image_name_prefix,
                                            self.scope)
 
-        pipe_mount = '%s:%s' % (self.paths.host_pipe_dir,
-                                self.paths.sandbox_pipe_dir)
-        storlet_mount = '%s:%s:ro' % (self.paths.host_storlet_base_dir,
-                                      self.paths.sandbox_storlet_base_dir)
-        storlet_native_lib_mount = '%s:%s:ro' % (
-            self.paths.host_storlet_native_lib_dir,
-            self.paths.sandbox_storlet_native_lib_dir)
-        storlet_native_bin_mount = '%s:%s:ro' % (
-            self.paths.host_storlet_native_bin_dir,
-            self.paths.sandbox_storlet_native_bin_dir)
+        mounts = [
+            DockerMount('/dev/log', '/dev/log', type='bind'),
+            DockerMount(self.paths.sandbox_pipe_dir,
+                        self.paths.host_pipe_dir,
+                        type='bind'),
+            DockerMount(self.paths.sandbox_storlet_base_dir,
+                        self.paths.host_storlet_base_dir,
+                        type='bind'),
+            DockerMount(self.paths.sandbox_storlet_native_lib_dir,
+                        self.paths.host_storlet_native_lib_dir,
+                        type='bind', read_only=True),
+            DockerMount(self.paths.sandbox_storlet_native_bin_dir,
+                        self.paths.host_storlet_native_bin_dir,
+                        type='bind', read_only=True)
+            ]
 
-        cmd = [os.path.join(self.paths.host_restart_script_dir,
-                            'restart_docker_container'),
-               docker_container_name, docker_image_name, pipe_mount,
-               storlet_mount, storlet_native_lib_mount,
-               storlet_native_bin_mount]
+        try:
+            client = docker.from_env()
+            # Stop the existing storlet container
+            try:
+                scontainer = client.containers.get(docker_container_name)
+            except docker.errors.NotFound:
+                # The container is not yet created
+                pass
+            else:
+                scontainer.stop(timeout=self.sandbox_stop_timeout)
 
-        proc = subprocess.Popen(cmd, stdout=subprocess.PIPE,
-                                stderr=subprocess.PIPE)
-        stdout, stderr = proc.communicate()
+            # Check whether a new container can be started
+            if self.max_containers_per_node > 0:
+                all_scontainers = client.containers.list(
+                    filters={'label': 'managed_by=storlets'})
+                if len(all_scontainers) >= self.max_containers_per_node:
+                    raise StorletRuntimeException(
+                        "Cannot start a container because of limit")
 
-        if stdout:
-            if not isinstance(stdout, str):
-                stdout = stdout.decode("utf-8")
-            self.logger.debug('STDOUT: %s' % stdout.replace('\n', '#012'))
-        if stderr:
-            if not isinstance(stderr, str):
-                stderr = stderr.decode("utf-8")
-            self.logger.error('STDERR: %s' % stderr.replace('\n', '#012'))
-
-        if proc.returncode:
-            raise StorletRuntimeException('Failed to restart docker container')
+            # Start the new one
+            client.containers.run(
+                docker_image_name, detach=True, name=docker_container_name,
+                network_disabled=True, mounts=mounts, user='swift',
+                auto_remove=True, stop_signal='SIGHUP',
+                labels={'managed_by': 'storlets'})
+        except docker.errors.ImageNotFound:
+            msg = "Image %s is not found" % docker_image_name
+            raise StorletRuntimeException(msg)
+        except docker.errors.APIError:
+            self.logger.exception("Failed to manage docker containers")
+            raise StorletRuntimeException("Docker runtime error")
 
     def restart(self):
         """
@@ -429,8 +451,8 @@ class RunTimeSandbox(object):
         if (storlet_daemon_status == -1):
             # We failed to send a command to the factory.
             # Best we can do is execute the container.
-            self.logger.debug('Failed to check Storlet daemon status, '
-                              'restart Docker container')
+            self.logger.debug('Failed to check the storlet daemon status. '
+                              'Restart Docker container')
             self.restart()
             storlet_daemon_status = 0
 
@@ -441,13 +463,15 @@ class RunTimeSandbox(object):
                               'is running. Stopping daemon')
             res = self.stop_storlet_daemon(sreq.storlet_main)
             if res != 1:
+                self.logger.warning('Failed to stop the storlet daemon. '
+                                    'Restart Docker container')
                 self.restart()
             else:
                 self.logger.debug('Deamon stopped')
             storlet_daemon_status = 0
 
         if (storlet_daemon_status == 0):
-            self.logger.debug('Going to start storlet daemon!')
+            self.logger.debug('Going to start the storlet daemon!')
 
             # TODO(takashi): This is not needed for python application
             classpath = self._get_storlet_classpath(
@@ -686,7 +710,12 @@ class StorletInvocationProtocol(object):
                         % self.task_id)
                     pass
 
-            six.reraise(exc_type, exc_value, exc_traceback)
+            if exc_value is None:
+                exc_value = exc_traceback
+            if exc_value.__traceback__ is not exc_traceback:
+                raise exc_value.with_traceback(exc_traceback)
+            raise exc_value
+
         if fd not in r:
             raise StorletRuntimeException('Read fd is not ready')
 
