@@ -13,7 +13,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from six.moves import StringIO
+from io import StringIO
 
 
 class FakeStorletFile(object):
