@@ -18,7 +18,6 @@ import os
 import random
 import string
 from swiftclient import client as c
-from nose.plugins.attrib import attr
 from contextlib import contextmanager
 from tests.functional.java import StorletJavaFunctionalTest
 import unittest
@@ -33,7 +32,6 @@ class TestIdentityStorlet(StorletJavaFunctionalTest):
         super(TestIdentityStorlet, self).setUp('IdentityStorlet',
                                                'identitystorlet-1.0.jar',
                                                main_class,
-                                               'myobjects',
                                                'source.txt',
                                                ['get42'],
                                                headers)
@@ -59,7 +57,7 @@ class TestIdentityStorlet(StorletJavaFunctionalTest):
         if op == 'GET':
             # Get original object
             original_h, original_c = c.get_object(self.url, self.token,
-                                                  'myobjects',
+                                                  self.container,
                                                   self.storlet_file,
                                                   response_dict=dict())
             # print original_headers
@@ -69,14 +67,14 @@ class TestIdentityStorlet(StorletJavaFunctionalTest):
                 self.container, self.storlet_file,
                 query_string=querystring, response_dict=dict(),
                 headers=headers, resp_chunk_size=file_length)
-            processed_c = ''
+            processed_c = b''
             for chunk in returned_c:
                 if chunk:
                     processed_c += chunk
 
             if params is not None and params.get('execute', None) is not None:
-                mdv = processed_h['X-Object-Meta-Execution result'.lower()]
-                self.assertEqual(mdv, '42')
+                mdv = processed_h['X-Object-Meta-Execution-Result'.lower()]
+                self.assertEqual('42', mdv)
             if params is not None and params.get('double', None) == 'true':
                 self.assertEqual(original_c, processed_c[:file_length])
                 self.assertEqual(original_c, processed_c[file_length:])
@@ -88,8 +86,9 @@ class TestIdentityStorlet(StorletJavaFunctionalTest):
         if op == 'PUT':
             # PUT a random file
             response = dict()
-            uploaded_c = ''.join(random.choice(string.ascii_uppercase +
-                                 string.digits) for _ in range(1024))
+            uploaded_c = ''.join(
+                random.choice(string.ascii_uppercase + string.digits)
+                for _ in range(1024)).encode('ascii')
             random_md = ''.join(random.choice(string.ascii_uppercase +
                                 string.digits) for _ in range(32))
             content_length = None
@@ -100,7 +99,7 @@ class TestIdentityStorlet(StorletJavaFunctionalTest):
                          "application/octet-stream", headers, None, None,
                          querystring, response)
             resp_headers, saved_c = c.get_object(self.url, self.token,
-                                                 'myobjects',
+                                                 self.container,
                                                  'identity_random_source',
                                                  response_dict=dict())
 
@@ -111,42 +110,45 @@ class TestIdentityStorlet(StorletJavaFunctionalTest):
                 assert(uploaded_c == saved_c)
 
             if params is not None and params.get('execute', None) is not None:
-                mdv = resp_headers['X-Object-Meta-Execution result'.lower()]
-                self.assertEqual(mdv, '42')
+                mdv = resp_headers['X-Object-Meta-Execution-Result'.lower()]
+                self.assertEqual('42', mdv)
 
-            self.assertEqual(resp_headers['X-Object-Meta-Testkey'.lower()],
-                             random_md)
+            self.assertEqual(random_md,
+                             resp_headers['X-Object-Meta-Testkey'.lower()])
 
     @contextmanager
     def _filecontext(self, path):
-        yield
         try:
-            os.remove(path)
-        except Exception:
-            pass
+            yield
+        finally:
+            try:
+                os.remove(path)
+            except Exception:
+                pass
 
-    @attr('slow')
-    def test_put_1GB_file(self):
-        gf_file_path = '/tmp/1GB_file'
+    def test_put_10MB_file(self):
+        gf_file_path = '/tmp/10MB_file'
         with self._filecontext(gf_file_path):
-            GBFile = open('/tmp/1GB_file', 'w')
-            for _ in range(128):
-                uploaded_content = ''.join('1' for _ in range(8 * 1024 * 1024))
-                GBFile.write(uploaded_content)
-            GBFile.close()
+            TenMBFile = open('/tmp/10MB_file', 'wb')
+            for _ in range(10):
+                uploaded_content = ''.join('1' for _ in range(1 * 1024 * 1024))
+                TenMBFile.write(uploaded_content.encode("utf-8"))
+            TenMBFile.close()
 
             headers = {'X-Run-Storlet': self.storlet_name}
             headers.update(self.additional_headers)
-            GBFile = open('/tmp/1GB_file', 'r')
+            TenMBFile = open('/tmp/10MB_file', 'rb')
             response = dict()
             c.put_object(self.url, self.token,
-                         self.container, '1GBFile', GBFile,
-                         1024 * 1024 * 1024, None, None,
+                         self.container, '10MBFile', TenMBFile,
+                         10 * 1024 * 1024, None, None,
                          "application/octet-stream",
                          headers, None, None, None, response)
             status = response.get('status')
             self.assertIn(status, [200, 201])
 
+    test_put_10MB_file.slow = 1
+
     def test_put(self):
         self.invoke_storlet('PUT')
 
@@ -177,26 +179,27 @@ class TestIdentityStorlet(StorletJavaFunctionalTest):
         srange = 'bytes=%d-%d' % (start, end)
         headers = {'X-Run-Storlet': self.storlet_name,
                    'X-Storlet-Range': srange}
+        headers.update(self.additional_headers)
         junk, content = c.get_object(self.url, self.token,
-                                     'myobjects',
+                                     self.container,
                                      'small',
                                      headers=headers,
                                      response_dict=dict())
-        self.assertEqual(content, expected)
+        self.assertEqual(expected, content)
 
     def test_get_ranges(self):
         response = dict()
         c.put_object(self.url, self.token,
                      self.container, 'small',
-                     '0123456789abcd',
+                     b'0123456789abcd',
                      response_dict=response)
 
-        self._test_get_range(0, 0, '0')
-        self._test_get_range(0, 10, '0123456789a')
-        self._test_get_range(2, 10, '23456789a')
-        self._test_get_range(10, 13, 'abcd')
-        self._test_get_range(10, 14, 'abcd')
-        self._test_get_range(10, 15, 'abcd')
+        self._test_get_range(0, 0, b'0')
+        self._test_get_range(0, 10, b'0123456789a')
+        self._test_get_range(2, 10, b'23456789a')
+        self._test_get_range(10, 13, b'abcd')
+        self._test_get_range(10, 14, b'abcd')
+        self._test_get_range(10, 15, b'abcd')
 
 
 class TestIdentityStorletOnProxy(TestIdentityStorlet):
