@@ -12,8 +12,8 @@
 # implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-import mock
 import unittest
+from unittest import mock
 
 from storlets.sbus import command as sbus_cmd
 from storlets.agent.daemon.server import StorletDaemon, StorletDaemonLoadError
@@ -71,6 +71,9 @@ class TestStorletDaemonMain(test_server.TestSBusServerMain):
         # so need more assertions.
         self._test_main_loop_stop(sbus_cmd.SBUS_CMD_CANCEL)
 
+    def test_main_loop_timeout(self):
+        self._test_main_loop_timeout()
+
 
 if __name__ == '__main__':
     unittest.main()
